@@ -3,22 +3,38 @@ import os
 import pickle
 import queue
 import threading
-import typing as t
 import warnings
 from copy import deepcopy
 from time import time
+from typing import Dict, List, Optional, Tuple, TYPE_CHECKING, Union
 
 from deepdiff import DeepDiff, Delta
+from lightning_utilities.core.apply_func import apply_to_collection
 
 import lightning_app
-from lightning_app.api.request_types import APIRequest, CommandRequest, DeltaRequest
-from lightning_app.core.constants import FLOW_DURATION_SAMPLES, FLOW_DURATION_THRESHOLD, STATE_ACCUMULATE_WAIT
+from lightning_app import _console
+from lightning_app.api.request_types import _APIRequest, _CommandRequest, _DeltaRequest
+from lightning_app.core.constants import (
+    DEBUG_ENABLED,
+    FLOW_DURATION_SAMPLES,
+    FLOW_DURATION_THRESHOLD,
+    FRONTEND_DIR,
+    STATE_ACCUMULATE_WAIT,
+)
 from lightning_app.core.queues import BaseQueue, SingleProcessQueue
+from lightning_app.core.work import LightningWork
 from lightning_app.frontend import Frontend
-from lightning_app.storage.path import storage_root_dir
-from lightning_app.utilities.app_helpers import _delta_to_app_state_delta, _LightningAppRef
+from lightning_app.storage import Drive, Path
+from lightning_app.storage.path import _storage_root_dir
+from lightning_app.utilities import frontend
+from lightning_app.utilities.app_helpers import (
+    _delta_to_app_state_delta,
+    _LightningAppRef,
+    _should_dispatch_app,
+    Logger,
+)
 from lightning_app.utilities.commands.base import _process_requests
-from lightning_app.utilities.component import _convert_paths_after_init
+from lightning_app.utilities.component import _convert_paths_after_init, _validate_root_flow
 from lightning_app.utilities.enum import AppStage, CacheCallsKeys
 from lightning_app.utilities.exceptions import CacheMissException, ExitAppException
 from lightning_app.utilities.layout import _collect_layout
@@ -27,17 +43,23 @@ from lightning_app.utilities.scheduler import SchedulerThread
 from lightning_app.utilities.tree import breadth_first
 from lightning_app.utilities.warnings import LightningFlowWarning
 
-if t.TYPE_CHECKING:
+if TYPE_CHECKING:
+    from lightning_app.core.flow import LightningFlow
     from lightning_app.runners.backends.backend import Backend, WorkManager
+    from lightning_app.runners.runtime import Runtime
 
-logger = logging.getLogger(__name__)
+
+logger = Logger(__name__)
 
 
 class LightningApp:
     def __init__(
         self,
-        root: "lightning_app.LightningFlow",
-        debug: bool = False,
+        root: Union["LightningFlow", "LightningWork"],
+        flow_cloud_compute: Optional["lightning_app.CloudCompute"] = None,
+        log_level: str = "info",
+        info: frontend.AppInfo = None,
+        root_path: str = "",
     ):
         """The Lightning App, or App in short runs a tree of one or more components that interact to create end-to-end
         applications. There are two kinds of components: :class:`~lightning_app.core.flow.LightningFlow` and
@@ -45,20 +67,28 @@ class LightningApp:
         created by other users.
 
         The Lightning App alternatively run an event loop triggered by delta changes sent from
-        either :class:`~lightning.app.core.work.LightningWork` or from the Lightning UI.
+        either :class:`~lightning_app.core.work.LightningWork` or from the Lightning UI.
         Once deltas are received, the Lightning App runs
-        the :class:`~lightning.app.core.flow.LightningFlow` provided.
+        the :class:`~lightning_app.core.flow.LightningFlow` provided.
 
         Arguments:
-            root: The root LightningFlow component, that defined all
-                the app's nested components, running infinitely.
-            debug: Whether to activate the Lightning Logger debug mode.
+            root: The root ``LightningFlow`` or ``LightningWork`` component, that defines all the app's nested
+                 components, running infinitely. It must define a `run()` method that the app can call.
+            flow_cloud_compute: The default Cloud Compute used for flow, Rest API and frontend's.
+            log_level: The log level for the app, one of [`info`, `debug`].
                 This can be helpful when reporting bugs on Lightning repo.
+            info: Provide additional info about the app which will be used to update html title,
+                description and image meta tags and specify any additional tags as list of html strings.
+            root_path: Set this to `/path` if you want to run your app behind a proxy at `/path` leave empty for "/".
+                For instance, if you want to run your app at `https://customdomain.com/myapp`,
+                set `root_path` to `/myapp`.
+                You can learn more about proxy `here <https://www.fortinet.com/resources/cyberglossary/proxy-server>`_.
+
 
         .. doctest::
 
-            >>> from lightning import LightningFlow, LightningApp
-            >>> from lightning.app.runners import MultiProcessRuntime
+            >>> from lightning_app import LightningFlow, LightningApp
+            >>> from lightning_app.runners import MultiProcessRuntime
             >>> class RootFlow(LightningFlow):
             ...     def run(self):
             ...         print("Hello World!")
@@ -69,33 +99,44 @@ class LightningApp:
             Hello World!
         """
 
+        self.root_path = root_path  # when running behind a proxy
+
+        from lightning_app.core.flow import _RootFlow
+
+        if isinstance(root, LightningWork):
+            root = _RootFlow(root)
+
+        _validate_root_flow(root)
         self._root = root
+        self.flow_cloud_compute = flow_cloud_compute or lightning_app.CloudCompute(name="flow-lite")
 
         # queues definition.
-        self.delta_queue: t.Optional[BaseQueue] = None
-        self.readiness_queue: t.Optional[BaseQueue] = None
-        self.api_response_queue: t.Optional[BaseQueue] = None
-        self.api_publish_state_queue: t.Optional[BaseQueue] = None
-        self.api_delta_queue: t.Optional[BaseQueue] = None
-        self.error_queue: t.Optional[BaseQueue] = None
-        self.request_queues: t.Optional[t.Dict[str, BaseQueue]] = None
-        self.response_queues: t.Optional[t.Dict[str, BaseQueue]] = None
-        self.copy_request_queues: t.Optional[t.Dict[str, BaseQueue]] = None
-        self.copy_response_queues: t.Optional[t.Dict[str, BaseQueue]] = None
-        self.caller_queues: t.Optional[t.Dict[str, BaseQueue]] = None
-        self.work_queues: t.Optional[t.Dict[str, BaseQueue]] = None
-        self.commands: t.Optional[t.List] = None
+        self.delta_queue: Optional[BaseQueue] = None
+        self.readiness_queue: Optional[BaseQueue] = None
+        self.api_response_queue: Optional[BaseQueue] = None
+        self.api_publish_state_queue: Optional[BaseQueue] = None
+        self.api_delta_queue: Optional[BaseQueue] = None
+        self.error_queue: Optional[BaseQueue] = None
+        self.request_queues: Optional[Dict[str, BaseQueue]] = None
+        self.response_queues: Optional[Dict[str, BaseQueue]] = None
+        self.copy_request_queues: Optional[Dict[str, BaseQueue]] = None
+        self.copy_response_queues: Optional[Dict[str, BaseQueue]] = None
+        self.caller_queues: Optional[Dict[str, BaseQueue]] = None
+        self.flow_to_work_delta_queues: Optional[Dict[str, BaseQueue]] = None
+        self.work_queues: Optional[Dict[str, BaseQueue]] = None
+        self.commands: Optional[List] = None
 
         self.should_publish_changes_to_api = False
         self.component_affiliation = None
-        self.backend: t.Optional[Backend] = None
+        self.backend: Optional["Backend"] = None
         _LightningAppRef.connect(self)
-        self.processes: t.Dict[str, WorkManager] = {}
-        self.frontends: t.Dict[str, Frontend] = {}
+        self.processes: Dict[str, "WorkManager"] = {}
+        self.frontends: Dict[str, Frontend] = {}
         self.stage = AppStage.RUNNING
         self._has_updated: bool = True
-        self._schedules: t.Dict[str, t.Dict] = {}
-        self.threads: t.List[threading.Thread] = []
+        self._schedules: Dict[str, Dict] = {}
+        self.threads: List[threading.Thread] = []
+        self.exception = None
 
         # NOTE: Checkpointing is disabled by default for the time being.  We
         # will enable it when resuming from full checkpoint is supported. Also,
@@ -116,12 +157,31 @@ class LightningApp:
         # is only available after all Flows and Works have been instantiated.
         _convert_paths_after_init(self.root)
 
-        if debug:
-            logging.getLogger().setLevel(logging.DEBUG)
+        if log_level not in ("debug", "info"):
+            raise Exception(f"Log Level should be in ['debug', 'info']. Found {log_level}")
+
+        # Lazily enable debugging.
+        if log_level == "debug" or DEBUG_ENABLED:
+            if not DEBUG_ENABLED:
+                os.environ["LIGHTNING_DEBUG"] = "2"
+            _console.setLevel(logging.DEBUG)
+
+        logger.debug(f"ENV: {os.environ}")
+
+        # update index.html,
+        # this should happen once for all apps before the ui server starts running.
+        frontend.update_index_file(FRONTEND_DIR, info=info, root_path=root_path)
+
+        if _should_dispatch_app():
+            os.environ["LIGHTNING_DISPATCHED"] = "1"
+            from lightning_app.runners import MultiProcessRuntime
+
+            MultiProcessRuntime(self).dispatch()
 
     def get_component_by_name(self, component_name: str):
         """Returns the instance corresponding to the given component name."""
-        from lightning_app.structures import Dict, List
+        from lightning_app.structures import Dict as LightningDict
+        from lightning_app.structures import List as LightningList
         from lightning_app.utilities.types import ComponentTuple
 
         if component_name == "root":
@@ -131,8 +191,8 @@ class LightningApp:
 
         current = self.root
         for child_name in component_name.split(".")[1:]:
-            if isinstance(current, (Dict, List)):
-                child = current[child_name] if isinstance(current, Dict) else current[int(child_name)]
+            if isinstance(current, (LightningDict, LightningList)):
+                child = current[child_name] if isinstance(current, LightningDict) else current[int(child_name)]
             else:
                 child = getattr(current, child_name, None)
             if not isinstance(child, ComponentTuple):
@@ -182,7 +242,7 @@ class LightningApp:
 
     @property
     def checkpoint_dir(self) -> str:
-        return os.path.join(storage_root_dir(), "checkpoints")
+        return os.path.join(_storage_root_dir(), "checkpoints")
 
     def remove_changes_(self, state):
         for _, child in state["flows"].items():
@@ -225,7 +285,7 @@ class LightningApp:
         return new_state
 
     @staticmethod
-    def get_state_changed_from_queue(q: BaseQueue, timeout: t.Optional[int] = None):
+    def get_state_changed_from_queue(q: BaseQueue, timeout: Optional[int] = None):
         try:
             delta = q.get(timeout=timeout or q.default_timeout)
             return delta
@@ -235,60 +295,59 @@ class LightningApp:
     def check_error_queue(self) -> None:
         exception: Exception = self.get_state_changed_from_queue(self.error_queue)
         if isinstance(exception, Exception):
+            self.exception = exception
             self.stage = AppStage.FAILED
 
     @property
-    def flows(self) -> t.List["lightning_app.LightningFlow"]:
+    def flows(self) -> List["LightningFlow"]:
         """Returns all the flows defined within this application."""
-        return [self.root] + self.root.get_all_children()
+        return [self.root] + list(self.root.flows.values())
 
     @property
-    def works(self) -> t.List["lightning_app.LightningWork"]:
+    def works(self) -> List[LightningWork]:
         """Returns all the works defined within this application."""
         return self.root.works(recurse=True)
 
     @property
-    def named_works(self) -> t.List[t.Tuple[str, "lightning_app.LightningWork"]]:
+    def named_works(self) -> List[Tuple[str, LightningWork]]:
         """Returns all the works defined within this application with their names."""
         return self.root.named_works(recurse=True)
 
-    def _collect_deltas_from_ui_and_work_queues(self) -> t.List[t.Union[Delta, APIRequest, CommandRequest]]:
+    def _collect_deltas_from_ui_and_work_queues(self) -> List[Union[Delta, _APIRequest, _CommandRequest]]:
         # The aggregation would try to get as many deltas as possible
         # from both the `api_delta_queue` and `delta_queue`
         # during the `state_accumulate_wait` time.
         # The while loop can exit sooner if both queues are empty.
 
-        should_get_delta_from_api = True
-        should_get_component_output = True
         deltas = []
+        api_or_command_request_deltas = []
         t0 = time()
 
         while (time() - t0) < self.state_accumulate_wait:
 
-            if self.api_delta_queue and should_get_delta_from_api:
-                delta_from_api: t.Union[DeltaRequest, APIRequest, CommandRequest] = self.get_state_changed_from_queue(
-                    self.api_delta_queue
-                )  # TODO: rename
-                if delta_from_api:
-                    if isinstance(delta_from_api, DeltaRequest):
-                        delta_from_api = delta_from_api.delta
-                    deltas.append(delta_from_api)
-                else:
-                    should_get_delta_from_api = False
+            # TODO: Fetch all available deltas at once to reduce queue calls.
+            delta: Optional[
+                Union[_DeltaRequest, _APIRequest, _CommandRequest, ComponentDelta]
+            ] = self.get_state_changed_from_queue(self.delta_queue)
+            if delta:
+                if isinstance(delta, _DeltaRequest):
+                    deltas.append(delta.delta)
+                elif isinstance(delta, ComponentDelta):
+                    logger.debug(f"Received from {delta.id} : {delta.delta.to_dict()}")
+                    work = None
+                    try:
+                        work = self.get_component_by_name(delta.id)
+                    except (KeyError, AttributeError) as e:
+                        logger.error(f"The component {delta.id} couldn't be accessed. Exception: {e}")
 
-            if self.delta_queue and should_get_component_output:
-                component_output: t.Optional[ComponentDelta] = self.get_state_changed_from_queue(self.delta_queue)
-                if component_output:
-                    logger.debug(f"Received from {component_output.id} : {component_output.delta.to_dict()}")
-                    work = self.get_component_by_name(component_output.id)
-                    new_work_delta = _delta_to_app_state_delta(self.root, work, deepcopy(component_output.delta))
-                    deltas.append(new_work_delta)
+                    if work:
+                        delta = _delta_to_app_state_delta(self.root, work, deepcopy(delta.delta))
+                        deltas.append(delta)
                 else:
-                    should_get_component_output = False
+                    api_or_command_request_deltas.append(delta)
 
-            # if both queues were found empties, should break the while loop.
-            if not should_get_delta_from_api and not should_get_component_output:
-                break
+        if api_or_command_request_deltas:
+            _process_requests(self, api_or_command_request_deltas)
 
         for delta in deltas:
             # When aggregating deltas from the UI and the Works, and over the accumulation time window,
@@ -304,13 +363,17 @@ class LightningApp:
     def maybe_apply_changes(self) -> bool:
         """Get the deltas from both the flow queue and the work queue, merge the two deltas and update the
         state."""
+        self._send_flow_to_work_deltas(self.state)
 
         deltas = self._collect_deltas_from_ui_and_work_queues()
 
         if not deltas:
+            # Path and Drive aren't processed by DeepDiff, so we need to convert them to dict.
+            last_state = apply_to_collection(self.last_state, (Path, Drive), lambda x: x.to_dict())
+            state = apply_to_collection(self.state, (Path, Drive), lambda x: x.to_dict())
             # When no deltas are received from the Rest API or work queues,
             # we need to check if the flow modified the state and populate changes.
-            deep_diff = DeepDiff(self.last_state, self.state, verbose_level=2)
+            deep_diff = DeepDiff(last_state, state, verbose_level=2)
             if deep_diff:
                 # TODO: Resolve changes with ``CacheMissException``.
                 # new_state = self.populate_changes(self.last_state, self.state)
@@ -320,19 +383,11 @@ class LightningApp:
 
         logger.debug(f"Received {[d.to_dict() for d in deltas]}")
 
-        # 1: Process the API / Command Requests first as they might affect the state.
-        state_deltas = []
-        for delta in deltas:
-            if isinstance(delta, (APIRequest, CommandRequest)):
-                _process_requests(self, delta)
-            else:
-                state_deltas.append(delta)
-
         # 2: Collect the state
         state = self.state
 
         # 3: Apply the state delta
-        for delta in state_deltas:
+        for delta in deltas:
             try:
                 state += delta
             except Exception as e:
@@ -433,9 +488,13 @@ class LightningApp:
 
             self._has_updated = False
 
+        self._on_run_end()
+
         return True
 
     def _update_layout(self) -> None:
+        import lightning_app
+
         if self.backend:
             self.backend.resolve_url(self, base_url=None)
 
@@ -473,16 +532,16 @@ class LightningApp:
                 return True
         return False
 
-    def state_dict(self) -> t.Dict:
+    def state_dict(self) -> Dict:
         return self.state
 
-    def load_state_dict(self, state: t.Dict) -> None:
+    def load_state_dict(self, state: Dict) -> None:
         self.set_state(state)
 
     def load_state_dict_from_checkpoint_dir(
         self,
         checkpoints_dir: str,
-        version: t.Optional[int] = None,
+        version: Optional[int] = None,
     ) -> None:
         if not os.path.exists(checkpoints_dir):
             raise FileNotFoundError(f"The provided directory `{checkpoints_dir}` doesn't exist.")
@@ -503,7 +562,7 @@ class LightningApp:
         state = pickle.load(open(checkpoint_path, "rb"))
         self.load_state_dict(state)
 
-    def _dump_checkpoint(self) -> t.Optional[str]:
+    def _dump_checkpoint(self) -> Optional[str]:
         checkpoints_dir = self.checkpoint_dir
         # TODO: Add supports to remotely saving checkpoints.
         if checkpoints_dir.startswith("s3:"):
@@ -527,7 +586,7 @@ class LightningApp:
             pickle.dump(self.state_dict(), f)
         return checkpoint_path
 
-    def connect(self, runtime: "lightning_app.runners.runtime.Runtime") -> None:
+    def connect(self, runtime: "Runtime") -> None:
         """Override to customize your application to the runtime."""
         pass
 
@@ -535,7 +594,7 @@ class LightningApp:
         if self._has_updated:
             self._update_layout()
 
-    def _register_schedule(self, schedule_hash: str, schedule_metadata: t.Dict) -> None:
+    def _register_schedule(self, schedule_hash: str, schedule_metadata: Dict) -> None:
         # create a thread only if a user uses the flow's schedule method.
         if not self._schedules:
             scheduler_thread = SchedulerThread(self)
@@ -550,3 +609,56 @@ class LightningApp:
         # disable any flow schedules.
         for flow in self.flows:
             flow._disable_running_schedules()
+
+    def _on_run_end(self):
+        if os.getenv("LIGHTNING_DEBUG") == "2":
+            del os.environ["LIGHTNING_DEBUG"]
+            _console.setLevel(logging.INFO)
+
+    @staticmethod
+    def _extract_vars_from_component_name(component_name: str, state):
+        child = state
+        for child_name in component_name.split(".")[1:]:
+            if child_name in child["flows"]:
+                child = child["flows"][child_name]
+            elif "structures" in child and child_name in child["structures"]:
+                child = child["structures"][child_name]
+            elif child_name in child["works"]:
+                child = child["works"][child_name]
+            else:
+                return None
+
+        # Note: Remove private keys
+        return {k: v for k, v in child["vars"].items() if not k.startswith("_")}
+
+    def _send_flow_to_work_deltas(self, state) -> None:
+        if not self.flow_to_work_delta_queues:
+            return
+
+        for w in self.works:
+            if not w.has_started:
+                continue
+
+            # Don't send changes when the state has been just sent.
+            if w.run.has_sent:
+                continue
+
+            state_work = self._extract_vars_from_component_name(w.name, state)
+            last_state_work = self._extract_vars_from_component_name(w.name, self._last_state)
+
+            # Note: The work was dynamically created or deleted.
+            if state_work is None or last_state_work is None:
+                continue
+
+            # Note: The flow shouldn't update path or drive manually.
+            last_state_work = apply_to_collection(last_state_work, (Path, Drive), lambda x: None)
+            state_work = apply_to_collection(state_work, (Path, Drive), lambda x: None)
+
+            deep_diff = DeepDiff(last_state_work, state_work, verbose_level=2).to_dict()
+
+            if "unprocessed" in deep_diff:
+                deep_diff.pop("unprocessed")
+
+            if deep_diff:
+                logger.debug(f"Sending deep_diff to {w.name} : {deep_diff}")
+                self.flow_to_work_delta_queues[w.name].put(deep_diff)
